#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

'''Integrate both Model Definition classes with Models.

'''

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from xoeuf.odoo import models
from xoutil.eight.meta import metaclass
from xoutil.future.types import SimpleNamespace

from . import get_modelname
from . import _proxy

__all__ = ['ImportNamespace']


class _ImportNamespaceType(type):
    def __new__(cls, name, bases, attrs):
        import warnings
        warnings.warn(
            'The entire xoeuf.models.importer is now deprecated. '
            'Please use xoeuf.models.proxy directly.',
            DeprecationWarning
        )
        definitions = SimpleNamespace()
        proxies = {}
        for attr, val in attrs.items():
            if isinstance(val, type) and issubclass(val, models.BaseModel):
                setattr(definitions, attr, val)
                proxies[attr] = _proxy.ModelProxy(get_modelname(val))
        attrs = dict(proxies, defs=definitions)
        return super(_ImportNamespaceType, cls).__new__(
            cls,
            name,
            bases,
            attrs
        )


class ImportNamespace(metaclass(_ImportNamespaceType)):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

{
    'name': 'test_view_model',
    'version': '1.0',
    'depends': ['base', 'test_view_model'],
    'auto_install': False,
    'installable': True
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
{
    'name': 'test_xoeuf_orm',
    'author': 'Merchise Autrement [~º/~]',
    'description': 'Test ORM',
    'depends': ['base', ],
    'data': ['data/nodes.xml'],
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from . import test_nodes_xml  # noqa



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

import contextlib

from hypothesis import strategies as s, given
from xoeuf import fields
from xoeuf.odoo.tests.common import TransactionCase

from ..models import COLORS, Pax, CARS, WORK_TYPE

colors = s.sampled_from(COLORS.__members__.values())
color_names = s.sampled_from(COLORS.__members__.keys())
cars = s.sampled_from(CARS.__members__.values())
paxs = s.sampled_from(Pax.__members__.values())
wtypes = s.sampled_from(WORK_TYPE.__members__.values())


class TestEnum(TransactionCase):
    at_install = False
    post_install = not at_install

    def setUp(self):
        super(TestEnum, self).setUp()
        self.EnumModel = self.env['test.enum.model']

    def test_column_type_force_char_columns(self):
        self.assertIsInstance(self.EnumModel._fields['color'], fields.Integer)
        self.assertIsInstance(self.EnumModel._fields['color2'], fields.Char)

    def test_can_set_valid_integers(self):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({'color': 1, 'car': CARS.FORD, 'pax': 1})
        id = obj.id
        self.EnumModel.invalidate_cache()
        obj = self.EnumModel.browse(id)
        assert obj.color == COLORS.Red, '%r == %r' % (obj.color, COLORS.Red)
        assert obj.car is CARS.FORD
        assert obj.pax == 1

    @given(colors)
    def test_can_set_valid_values(self, color):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({'color': color})
        id = obj.id
        self.EnumModel.invalidate_cache()
        obj = self.EnumModel.browse(id)
        assert obj.color == color, '%r == %r' % (obj.color, color)

    @given(colors)
    def test_can_set_valid_values_as_integers_and_get_values(self, color):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({'color': int(color)})
        id = obj.id
        self.EnumModel.invalidate_cache()
        obj = self.EnumModel.browse(id)
        assert obj.color == color, '%r == %r' % (obj.color, color)
        assert isinstance(obj.color, COLORS)

    def test_cannot_set_invalid_integers(self):
        # Sinces tests are run while the registry is being populated, i.e not
        # ready, we need to trick it to allow receivers be executed.
        with force_ready(self.env.registry), self.assertRaises(ValueError):
            self.EnumModel.create({'color': 10})

    def test_cannot_write_invalid_integers(self):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({'color': 1})
            with self.assertRaises(ValueError):
                obj.write({'color': 10})

    def test_cannot_write_invalid_values(self):
        obj = self.EnumModel.create({'car': CARS.FORD})
        with force_ready(self.env.registry), self.assertRaises(ValueError):
            obj.write({'car': 'any other brand'})

    def test_search_non_integer(self):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({'car': CARS.FORD})
            self.EnumModel.invalidate_cache()
            self.assertEqual(
                self.EnumModel.search([('car', '=', CARS.FORD)]),
                obj
            )
            self.assertEqual(
                self.EnumModel.search([('car', '=', CARS.CHEV)], count=True),
                0
            )
            self.assertEqual(
                self.EnumModel.search([('car', 'in', (CARS.FORD, CARS.CHEV))]),
                obj
            )
            with self.assertRaises(ValueError):
                self.EnumModel.search([('car', '=', 1)])

    def test_performance(self):
        from xoeuf.fields.enumeration import EnumerationAdapter
        self.assertIn(EnumerationAdapter, type(self.EnumModel).mro())
        self.assertNotIn(EnumerationAdapter, type(self.env['res.partner']).mro())

    def test_search_integer(self):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({'color': COLORS.Red})
            self.EnumModel.invalidate_cache()
            self.assertEqual(
                self.EnumModel.search([('color', '=', 1)]),
                obj
            )
            with self.assertRaises(ValueError):
                self.EnumModel.search([('color', '=', 10)])

    def test_color2_computed_field_read(self):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({'color2': COLORS.Red})
            self.assertEqual(obj.color2_name, 'Red')

    def test_color3_default_value(self):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({})
            self.assertEqual(obj.color3, COLORS.Red)

    @given(color_names)
    def test_color2_computed_field_set_on_create(self, name):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({'color2_name': name})
            self.assertEqual(obj.color2, COLORS.__members__[name])

    @given(color_names)
    def test_color2_computed_field_set_on_assignment(self, name):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({})
            obj.color2_name = name
            self.assertEqual(obj.color2, COLORS.__members__[name])

    @given(color_names)
    def test_color2_computed_field_set_on_write(self, name):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({})
            obj.write({'color2_name': name})
            self.assertEqual(obj.color2, COLORS.__members__[name])

    def test_color_computed_field_read(self):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({'color': COLORS.Red})
            self.assertEqual(obj.color_name, 'Red')

    @given(color_names)
    def test_color_computed_field_set_on_create(self, name):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({'color_name': name})
            self.assertEqual(obj.color, COLORS.__members__[name])

    @given(color_names)
    def test_color_computed_field_set_on_assignment(self, name):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({})
            obj.color_name = name
            self.assertEqual(obj.color, COLORS.__members__[name])

    @given(color_names)
    def test_color_computed_field_set_on_write(self, name):
        with force_ready(self.env.registry):
            obj = self.EnumModel.create({})
            obj.write({'color_name': name})
            self.assertEqual(obj.color, COLORS.__members__[name])

    @given(wtypes)
    def test_intsubclasses_values(self, work_type):
        from xoeuf.fields.enumeration import _get_db_value
        value = _get_db_value(self.EnumModel._fields['wtype'], work_type)
        self.assertIs(value, work_type)


@contextlib.contextmanager
def force_ready(registry):
    ready = registry.ready
    registry.ready = True
    try:
        yield
    finally:
        registry.ready = ready



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

import unittest
from xoeuf.odoo.tests.common import TransactionCase


class TestLocalizedDt(TransactionCase):

    @unittest.expectedFailure
    def test_getting_the_field(self):
        obj = self.env['test_localizated_dt.model'].create(
            {'dt_at_tzone': '2018-01-01 12:00:00'}
        )
        # 12M on January (no DTS) in Cuba is -5 hours from UTC
        self.assertEqual(obj.dt, '2018-01-01 17:00:00')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)
from xoutil.future.datetime import TimeSpan, parse_date

from datetime import date
from hypothesis import strategies, given
from xoeuf.odoo.tests.common import TransactionCase


maybe_dates = strategies.dates(min_value=date(1900, 1, 1)) | strategies.none()


@strategies.composite
def timespans(draw):
    start = draw(maybe_dates)
    end = draw(maybe_dates)
    return TimeSpan(start, end)


class TestTimespan(TransactionCase):
    def setUp(self):
        super(TestTimespan, self).setUp()
        self.Value = self.env['test.timespan.value']

    def assertDateEqual(self, value1, value2):
        if value1 and value2:
            if isinstance(value1, date):
                v1 = value1
                # Ensure we're using date1 with the Python's date.  See
                # xoutils bug:
                # https://gitlab.lahavane.com/merchise/xoutil/issues/3
                date1 = date(value1.year, value1.month, value1.day)
            else:
                v1 = parse_date(value1)
                date1 = value1
            if isinstance(value2, date):
                v2 = value2
                date2 = date(value2.year, value2.month, value2.day)
            else:
                v2 = parse_date(value2)
                date2 = value2
            self.assertTrue(
                v1 == v2,
                '%r is not equal to %r' % (date1, date2)
            )
        else:
            return value1 in (False, None) and value2 in (False, None)

    @given(timespans())
    def test_create(self, value):
        obj = self.Value.create({
            'period': value
        })
        self.assertDateEqual(obj.start_date, value.start_date)
        self.assertDateEqual(obj.end_date, value.end_date)

    @given(maybe_dates, maybe_dates)
    def test_create2(self, start, end):
        obj = self.Value.create({
            'start_date': start,
            'end_date': end,
        })
        self.assertEqual(obj.period, TimeSpan(start, end))

    @given(timespans())
    def test_set(self, value):
        obj = self.Value.create({})
        obj.period = value
        self.assertDateEqual(obj.start_date, value.start_date)
        self.assertDateEqual(obj.end_date, value.end_date)

    @given(maybe_dates, maybe_dates)
    def test_set2(self, start, end):
        obj = self.Value.create({})
        obj.start_date = start
        obj.end_date = end
        self.assertEqual(obj.period, TimeSpan(start, end))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from xoeuf.odoo.tests.common import TransactionCase


class TestViewModel(TransactionCase):

    def test_shared_extensions(self):
        self.assertTrue(all(
            'new_value' in self.env[model]._fields
            for model in ['test_view_model.model', 'view_model1', 'view_model2']
        ))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

'''Xœuf tools for Open Object (OpenERP) models.

'''
from __future__ import (division as _py3_division,
                        print_function as _py3_print,
                        absolute_import as _py3_abs_import)

from typing import Union  # noqa
from datetime import datetime as _dt, date as _d, time as _t
from xoutil.names import nameof

try:
    from xoeuf.odoo.tools import safe_eval, float_round  # noqa: reexport
except ImportError:
    # This allows to generate the documentation without actually installing
    # Odoo
    pass


try:
    from xoeuf.odoo.tools import (
        DEFAULT_SERVER_DATE_FORMAT as _SVR_DATE_FMT,
        DEFAULT_SERVER_DATETIME_FORMAT as _SVR_DATETIME_FMT,
    )
except ImportError:
    # This allows to generate the documentation without actually installing
    # Odoo
    _SVR_DATE_FMT = '%Y-%m-%d'
    _SVR_DATETIME_FMT = '%Y-%m-%d %H:%M:%S'

import pytz

utc = pytz.UTC

_SVR_DATETIME_FMT2 = _SVR_DATETIME_FMT + '.%f'


def strip_tzinfo(dt):
    # type: (datetime) -> datetime
    '''Return the given datetime value with tzinfo removed.

    .. deprecated:: 0.50.0  Use the replace method of datetime.

    '''
    return dt.replace(tzinfo=None)


def localize_datetime(self, datetime_value=None, from_tz='UTC', to_tz='UTC'):
    """Convert datetime value from a timezone to another.

    We assume `datetime_value` is expressed in the timezone given in
    `from_tz`, i.e is a naive datetime.

    If `datetime_value` is None then context today is used.

    If `from_tz` or `to_tz` is None then user timezone is used.

    If `from_tz` is the same as `to_tz`, return `datetime_value`.

    """
    from xoeuf.odoo import fields
    if not from_tz:
        from_tz = self.env.user.tz or 'UTC'
    if not to_tz:
        to_tz = self.env.user.tz or 'UTC'
    if not datetime_value:
        if from_tz != 'UTC':
            datetime_value = fields.Date.context_today(self)
        else:
            datetime_value = fields.Datetime.now()
    return localtime_as_remotetime(datetime_value, from_tz, to_tz)


def date2str(d):
    # type: (date) -> str
    '''Convert a date to a string using `OpenERP` default date format.

    If the argument is not a `datetime.date`:class:, normalize it first with
    `normalize_datetime`:func:.

    '''
    if not isinstance(d, _d):
        d = normalize_datetime(d)
    return d.strftime(_SVR_DATE_FMT)


normalize_datestr = date2str


def dt2str(dt):
    # type: (datetime) -> str
    '''Convert a date-time to a string using `OpenERP` default datetime
    format.

    If the argument is not a `datetime.datetime`:class:, normalize it first
    with `normalize_datetime`:func:.

    '''
    if not isinstance(dt, _dt):
        dt = normalize_datetime(dt)
    return dt.strftime(_SVR_DATETIME_FMT)


normalize_datetimestr = dt2str


def str2dt(s):
    # type: (str) -> datetime
    'Convert a string to a date-time using `OpenERP` default datetime format.'
    try:
        return _dt.strptime(s, _SVR_DATETIME_FMT)
    except ValueError:
        # Try a second time but allowing microseconds, this avoid some errors
        # when you save a .now() directly via the ORM.  It seems to not
        # sanitize properly the datetimes.
        return _dt.strptime(s, _SVR_DATETIME_FMT2)


parse_datetime = str2dt


def str2date(s):
    # type: (str) -> date
    'Convert a string to a date-time using `OpenERP` default date format.'
    return _dt.strptime(s, _SVR_DATE_FMT)


parse_date = str2date


def normalize_datetime(which):
    # type: (Any) -> datetime
    '''Normalizes `which` to a datetime.

    If `which` is a `datetime`, we ensure it will yield a valid string
    (matches the OpenERP datetime format).

    If `which` is a `date` is returned as a `datetime` with time components
    set to 0.  Otherwise, it must be a string with either of OpenERP's date or
    date-time format.  The date-time format is used first.

    For instance, having ``now`` and ``today`` values like::

       >>> import datetime
       >>> from xoutil.objects import validate_attrs
       >>> now = datetime.datetime(2014, 12, 20, 16, 0, 17, 678233)
       >>> today = now.date()

    Then, ``now`` is returned as-is::

       >>> normalize_datetime(now)
       datetime.datetime(2014, 12, 20, 16, 0, 17)

    But ``today`` is converted to a `datetime` with the same date components::

       >>> dtoday = normalize_datetime(today)
       >>> validate_attrs(today, dtoday,
       ...                force_equals=('year', 'month', 'day'))
       True

    If a string is given, a `datetime` is returned::

       >>> normalize_datetime('2014-02-12')
       datetime.datetime(2014, 2, 12, 0, 0)

    If the string does not match any of the server's date or datetime format,
    raise a ValueError::

       >>> normalize_datetime('not a datetime')  # doctest: +ELLIPSIS
       Traceback (most recent call last):
          ...
       ValueError: ...

    '''
    from xoutil.eight import string_types
    if isinstance(which, _dt):
        return str2dt(dt2str(which))
    elif isinstance(which, _d):
        return _dt(which.year, which.month, which.day)
    elif isinstance(which, string_types):
        try:
            return parse_datetime(which)
        except ValueError:
            return parse_date(which)
    else:
        raise TypeError("Expected a string, date or date but a '%s' was given"
                        % type(which))


def normalize_date(which):
    # type: (Any) -> date
    '''Normalizes `which` to a date.

    If `which` is a `date` is returned unchanged.  If is a `datetime`, then
    its `~datetime.date`:func: method is used.  Otherwise, it must be a string
    with either of OpenERP's date or date-time format.  The date format is
    tried first.

    For instance, having ``now`` and ``today`` values like::

       >>> import datetime
       >>> now = datetime.datetime.now()
       >>> today = now.date()

    Then, ``today`` is returned as-if::

       >>> normalize_date(today) is today
       True

    But ``now`` is converted to a `date`

       >>> normalize_date(now) == today
       True

    If a string is given, a `date` is returned::

       >>> normalize_date('2014-02-12 10:00:00')
       datetime.date(2014, 2, 12)

    If the string does not match any of the server's date or datetime format,
    raise a ValueError::

       >>> normalize_date('not a date')  # doctest: +ELLIPSIS
       Traceback (most recent call last):
          ...
       ValueError: ...

    '''
    from xoutil.eight import string_types
    if isinstance(which, _dt):
        return which.date()
    elif isinstance(which, _d):
        return which
    elif isinstance(which, string_types):
        try:
            return parse_date(which).date()
        except ValueError:
            return parse_datetime(which).date()
    else:
        raise TypeError("Expected a string, date or date but a '%s' was given"
                        % type(which))


def dt_as_timezone(dt, tz_name=None):
    # type: (datetime, str) -> datetime
    """ Localize datetime in desired timezone.

    :param datetime dt: datetime
    :param string tz_name: name of the timezone to localize the datetime
    :return: datetime with tzinfo, UTC in case tz_name is none.
    """
    dt = normalize_datetime(dt)
    if tz_name:
        tz = pytz.timezone(tz_name)
    else:
        tz = pytz.UTC
    return tz.localize(dt.replace(tzinfo=None))


def localtime_as_remotetime(dt_UTC, from_tz=utc, as_tz=utc, ignore_dst=False):
    # type: (datetime, Union[str, tzinfo], Union[str, tzinfo], bool) -> datetime
    """Compute the datetime as the timezone source, then force to it the desired
    TZ and back to UTC.

    :param dt_UTC: datetime in UTC

    :param from_tz: timezone to compute the datetime

    :param as_tz: timezone to localize the datetime

    :param ignore_dst: value used for time disambiguation. Is used just
                       when `dt_UTC` is an ambiguous or missing datetime
                       value.

    When clocks are moved back, we say that a fold is created in time.
    When the clocks are moved forward, a gap is created. A local time that
    falls in the fold is called ambiguous. A local time that falls in the
    gap is called missing.

    :return: datetime in desired timezone

    .. versionchanged:: 0.49.0 Added parameter `ignore_dst`.

    """
    dt_UTC = normalize_datetime(dt_UTC)
    if not isinstance(from_tz, pytz.tzinfo.tzinfo):
        from_tz = pytz.timezone(from_tz)
    if not isinstance(as_tz, pytz.tzinfo.tzinfo):
        as_tz = pytz.timezone(as_tz)
    if from_tz == as_tz:
        return dt_UTC
    local_timestamp = from_tz.localize(dt_UTC, is_dst=ignore_dst)
    return local_timestamp.astimezone(as_tz).replace(tzinfo=None)


def get_time_from_float(value):
    """Get time tuple ``(h, m, s)`` from a float `value`.

    """
    hours, minutes = divmod(value * 60, 60)
    minutes, seconds = divmod(minutes * 60, 60)
    return _t(hour=int(hours), minute=int(minutes), second=int(seconds))


def get_time_string(time, up_24=True, include_seconds=False):
    """ Get a human friendly representation of `time`.

    :param time_value: time value.

    :param up_24: True to use 24 hours time format.

    :param include_seconds: True to include seconds value in result.

    :return: string time formatted like `13:01 PM`.

    Examples:

       >>> import datetime
       >>> now = datetime.time(hour=13, minute=24, second=14)

       >>> get_time_string(now)
       '13:24'

       >>> get_time_string(now, include_seconds=True)
       '13:24:14'

       >>> get_time_string(now, up_24=False)
       '01:24 PM'

       >>> get_time_string(now, up_24=False, include_seconds=True)
       '01:24:14 PM'

    """
    if up_24:
        result = time.strftime('%T' if include_seconds else '%R')
    else:
        result = time.strftime('%r' if include_seconds else '%I:%M %p')
    return result


def get_time_string_from_float(value, up_24=True, include_seconds=False):
    """ Get a human friendly representation of time from a float value.

    :param value: float value that represent a time value.

    :param up_24: True for military time format

    :param include_seconds: True to include seconds value in result

    :return: string time formatted like `13:01 PM`

    Examples:

       >>> value = 13.404

       >>> get_time_string_from_float(value)
       '13:24'

       >>> get_time_string_from_float(value, include_seconds=True)
       '13:24:14'

       >>> get_time_string_from_float(value, up_24=False)
       '01:24 PM'

       >>> get_time_string_from_float(value, False, True)
       '01:24:14 PM'

    """
    return get_time_string(
        get_time_from_float(value),
        up_24=up_24,
        include_seconds=include_seconds
    )


_SAFE_EVAL_SYMBOLS = {}


def add_symbols_to_xmls(*objs, **symbols):
    '''Allow to use the provided symbols in XMLs.

    Technically this replaces the function `safe_eval` in module
    `odoo.tools.convert` to include the symbols in the context.

    We keep a global dict of symbols, and only replace the `safe_eval`
    function once and update the global dict

    :param objs: Objects to be made available under its `proper name
                 <xoutil.names.nameof>`:func:.

    :param symbols: Objects to be made available under the keyword argument.

    .. versionadded:: 0.42.0

    .. versionchanged:: 0.43.0 Added the `objs` variable positional arguments.

    '''
    if objs:
        symbols.update({nameof(obj, inner=True, full=False) for obj in objs})
    _SAFE_EVAL_SYMBOLS.update(symbols)


# HACK to make TERM_RELATIONSHIP_KIND available in XMLs.
from xoeuf.odoo.tools import convert
_safe_eval = convert.safe_eval


def custom_safe_eval(expr, ctx={}):
    ctx = dict(ctx)
    ctx.update(_SAFE_EVAL_SYMBOLS)
    return _safe_eval(expr, ctx)


convert.safe_eval = custom_safe_eval
